@@ -12,20 +12,54 @@ import matplotlib
 import matplotlib.pyplot as plt
 import seaborn as sns
 import pingouin as pg
+from tqdm import tqdm as _tqdm
 
 from seaborn_extensions.types import DataFrame, Axis, Figure, Iterables
 from seaborn_extensions.utils import get_grid_dims, filter_kwargs_by_callable
 
 
+"""
+import pandas as pd
+import pingouin as pg
+from seaborn_extensions import swarmboxenplot
+# Demo with various tests available in Pingouin:
+data = pd.DataFrame(
+    {"cont": np.random.random(20), "cat": np.random.choice(["a", "b"], 20)}
+)
+data.loc[data["cat"] == "b", "cont"] *= 5
+fig, stats = swarmboxenplot(data=data, x='cat', y='cont')
+data['h'] = ['cl_1'] * 10 + ['cl_2'] * 10
+fig, stats = swarmboxenplot(data=data, x='cat', y='cont', hue='h')
+
+data['cont1'] = data['cont'] + np.random.random(20)
+data['cont2'] = data['cont'] + np.random.random(20)
+fig, stats = swarmboxenplot(data=data, x='cat', y=['cont1', 'cont2'], hue='h')
+
+x = 'cat'
+y = 'cont'
+pg.ttest(*data.groupby(x)[y].apply(lambda x: list(x)))
+pg.mwu(*data.groupby(x)[y].apply(lambda x: list(x)))
+pg.kruskal(data=data, between=x, dv=y)
+pg.pairwise_ttests(data=data, between=x, dv=y, parametric=True)  # same as T-test
+pg.pairwise_ttests(data=data, between=x, dv=y, parametric=False)  # same as MWU
+
+c = data[x].astype(pd.CategoricalDtype())
+pg.linear_regression(pd.concat((c[col].cat.codes.rename(col) for col in c), axis=1), data[y])
+pg.linear_regression(pd.get_dummies(c), data[y])
+pg.logistic_regression(data[y], c.cat.codes)
+"""
+
+
 def swarmboxenplot(
     data: DataFrame,
     x: str,
     y: tp.Union[str, Iterables],
-    hue: str = None,
+    hue: tp.Optional[str] = None,
     swarm: bool = True,
     boxen: bool = True,
     bar: bool = False,
     orient: str = "vertical",
+    plot: bool = True,
     ax: tp.Union[Axis, tp.Sequence[Axis]] = None,
     test: tp.Union[bool, str] = "mann-whitney",
     to_test: str = "all",
@@ -33,9 +67,10 @@ def swarmboxenplot(
     test_upper_threshold: float = 0.05,
     test_lower_threshold: float = 0.01,
     plot_non_significant: bool = False,
-    plot_kws: tp.Dict[str, tp.Any] = None,
-    test_kws: tp.Dict[str, tp.Any] = None,
-    fig_kws: tp.Dict[str, tp.Any] = None,
+    plot_kws: tp.Optional[tp.Dict[str, tp.Any]] = None,
+    test_kws: tp.Optional[tp.Dict[str, tp.Any]] = None,
+    fig_kws: tp.Optional[tp.Dict[str, tp.Any]] = None,
+    tqdm: tp.Union[bool, tp.Dict[str, tp.Any]] = True,
 ) -> tp.Optional[tp.Union[Figure, DataFrame, tp.Tuple[Figure, DataFrame]]]:
     """
     A categorical plot that overlays individual observations
@@ -97,7 +132,10 @@ def swarmboxenplot(
     plot_kws: dict
         Additional values to pass to seaborn.boxenplot or seaborn.swarmplot
     test_kws: dict
-        Additional values to pass to pingouin.pairwise_ttests.
+        Additional values to pass to pingouin.pairwise_tests.
+        The default is: dict(parametric=False) to run a non-parametric test.
+    tqdm: bool, dict
+        Additional values to pass to pingouin.pairwise_tests.
         The default is: dict(parametric=False) to run a non-parametric test.
 
     Returns
@@ -132,28 +170,42 @@ def swarmboxenplot(
     if test_kws is None:
         test_kws = dict()
     if plot_kws is None:
-        plot_kws = dict()
+        plot_kws = dict(palette="tab10")
+    if isinstance(tqdm, bool):
+        tqdm_kws = dict(
+            disable=not tqdm, total=len(y) if not isinstance(y, str) else 1, desc="y"
+        )
+    else:
+        tqdm_kws = tqdm
+        tqdm_kws["disable"] = False
+        kw = dict(total=len(y) if not isinstance(y, str) else 1, desc="y")
+        for k, v in kw.items():
+            if k not in tqdm_kws:
+                tqdm_kws[k] = v
 
     data = data.sort_values([x] + ([hue] if hue is not None else []))
 
     if isinstance(y, (list, pd.Series, pd.Index)):
-        # TODO: display only one legend for hue
-        if ax is None:
-            n, m = get_grid_dims(y)
-            default_fig_kws = dict(
-                nrows=n, ncols=m, figsize=(m * 4, n * 4), sharex=True, squeeze=False
-            )
-            default_fig_kws.update(fig_kws or {})
-            fig, axes = plt.subplots(**default_fig_kws)
-            axes = axes.flatten()
-        elif isinstance(ax, np.ndarray):
-            axes = ax.flatten()
-        elif isinstance(ax, matplotlib.axes.Axes):
-            axes = np.asarray([ax])
+        if plot:
+            # TODO: display only one legend for hue
+            if ax is None:
+                n, m = get_grid_dims(y)
+                default_fig_kws = dict(
+                    nrows=n, ncols=m, figsize=(m * 4, n * 4), sharex=True, squeeze=False
+                )
+                default_fig_kws.update(fig_kws or {})
+                fig, axes = plt.subplots(**default_fig_kws)
+                axes = axes.flatten()
+            elif isinstance(ax, np.ndarray):
+                axes = ax.flatten()
+            elif isinstance(ax, matplotlib.axes.Axes):
+                axes = np.asarray([ax])
+        else:
+            axes = [None] * len(y)
 
         _stats = list()
         idx = -1
-        for idx, _var in enumerate(y):
+        for idx, _var in _tqdm(enumerate(y), **tqdm_kws):
             _ax = axes[idx]
             s: DataFrame = swarmboxenplot(
                 data=data,
@@ -164,6 +216,7 @@ def swarmboxenplot(
                 boxen=boxen,
                 bar=bar,
                 orient=orient,
+                plot=plot,
                 ax=_ax,
                 test=test,
                 to_test=to_test,
@@ -174,34 +227,31 @@ def swarmboxenplot(
                 plot_kws=plot_kws,
                 test_kws=test_kws,
             )
-            _ax.set(title=_var + _ax.get_title(), xlabel=None, ylabel=None)
+            if plot:
+                _ax.set(title=_var + _ax.get_title(), xlabel=None, ylabel=None)
             if test is not False:
                 _stats.append(s.assign(Variable=_var))
         # "close" excess subplots
-        for _ax in axes[idx + 1 :]:
-            _ax.axis("off")
+        if plot:
+            for _ax in axes[idx + 1 :]:
+                _ax.axis("off")
         if test is not False:
             stats = pd.concat(_stats).reset_index(drop=True)
             cols = [c for c in stats.columns if c != "Variable"]
             stats = stats.reindex(["Variable"] + cols, axis=1)
 
             # If there is just one test per `y` (no hue), correct p-values
-            if stats.shape == len(y):
+            if stats.shape[0] == len(y):
                 stats["p-cor"] = pg.multicomp(
-                    stats["p-unc"].values, method=multiple_testing
+                    stats["p-unc"].tolist(), method=multiple_testing
                 )[1]
         if ax is None:
-            return (fig, stats) if test else fig
+            return stats if not plot else (fig, stats) if test else fig
         return stats if test else None
 
     if data[y].dtype.name in ["category", "string", "object"]:
         raise ValueError("`y` variable must be numeric.")
 
-    if ax is None:
-        fig, _ax = plt.subplots(1, 1, figsize=(4, 4))
-    else:
-        _ax = ax
-
     horizontal = orient in ["horizontal", "horiz", "h"]
     if horizontal:
         x2 = y
@@ -210,31 +260,48 @@ def swarmboxenplot(
         y = y2
 
     # Plot vanilla seaborn
-    if boxen:
-        assert not bar
-        # Tmp fix for lack of support for Pandas Int64 in boxenplot:
-        if data[y].dtype.name == "Int64":
-            data[y] = data[y].astype(float)
-        boxen_kws = filter_kwargs_by_callable(plot_kws, sns.boxenplot)
-        sns.boxenplot(data=data, x=x, y=y, hue=hue, ax=_ax, **boxen_kws)
-    if bar:
-        assert not boxen
-        bar_kws = filter_kwargs_by_callable(plot_kws, sns.barplot)
-        sns.barplot(data=data, x=x, y=y, hue=hue, ax=_ax, **bar_kws)
+    if plot:
+        if ax is None:
+            default_fig_kws = dict(figsize=(4, 4))
+            default_fig_kws.update(fig_kws or {})
+            fig, _ax = plt.subplots(**default_fig_kws)
+        else:
+            _ax = ax
 
-    if (boxen or bar) and swarm:
-        _add_transparency_to_plot(_ax, kind="bar" if bar else "boxen")
-    if swarm:
-        swarm_kws = filter_kwargs_by_callable(plot_kws, sns.swarmplot)
-        if hue is not None and "dodge" not in swarm_kws:
-            swarm_kws["dodge"] = True
-        with warnings.catch_warnings():
-            warnings.filterwarnings("ignore", category=UserWarning)
-            sns.swarmplot(data=data, x=x, y=y, hue=hue, ax=_ax, **swarm_kws)
-    if horizontal:
-        _ax.set_yticklabels(_ax.get_yticklabels(), rotation=0, ha="right")
-    else:
-        _ax.set_xticklabels(_ax.get_xticklabels(), rotation=90, ha="right")
+        if boxen:
+            assert not bar
+            # Tmp fix for lack of support for Pandas Int64 in boxenplot:
+            if data[y].dtype.name == "Int64":
+                data[y] = data[y].astype(float)
+            boxen_kws = filter_kwargs_by_callable(plot_kws, sns.boxenplot)
+            sns.boxenplot(data=data, x=x, y=y, hue=hue, ax=_ax, **boxen_kws)
+        if bar:
+            assert not boxen
+            bar_kws = filter_kwargs_by_callable(plot_kws, sns.barplot)
+            sns.barplot(data=data, x=x, y=y, hue=hue, ax=_ax, **bar_kws)
+
+        if (boxen or bar) and swarm:
+            _add_transparency_to_plot(_ax, kind="bar" if bar else "boxen")
+        if swarm:
+            swarm_kws = filter_kwargs_by_callable(plot_kws, sns.swarmplot)
+            if hue is not None and "dodge" not in swarm_kws:
+                swarm_kws["dodge"] = True
+            with warnings.catch_warnings():
+                warnings.filterwarnings("ignore", category=UserWarning)
+                sns.swarmplot(
+                    data=data,
+                    x=x,
+                    y=y,
+                    # use `hue` as `x` to have scatter colored accordingly
+                    hue=hue if hue is not None else x,
+                    legend="auto" if hue is not None else False,
+                    ax=_ax,
+                    **swarm_kws,
+                )
+        if horizontal:
+            _ax.set_yticklabels(_ax.get_yticklabels(), rotation=0, ha="right")
+        else:
+            _ax.set_xticklabels(_ax.get_xticklabels(), rotation=90, ha="right")
 
     if test is False:
         return fig if ax is None else None
@@ -247,7 +314,7 @@ def swarmboxenplot(
         y = y2
 
     if test in [True, "t-test", "mann-whitney"]:
-        test_function = pg.pairwise_ttests
+        test_function = pg.pairwise_tests
         if test == "mann-whitney":
             test_kws["parametric"] = False
     elif test in ["kruskal"]:
@@ -266,8 +333,9 @@ def swarmboxenplot(
     datat = datat.loc[datat[x].isin(keep), :]
     if datat[x].dtype.name == "category":
         datat[x] = datat[x].cat.remove_unused_categories()
-    ylim = _ax.get_ylim()  # save original axis boundaries for later
-    ylength = abs(ylim[1]) + (abs(ylim[0]) if ylim[0] < 0 else 0)
+    if plot:
+        ylim = _ax.get_ylim()  # save original axis boundaries for later
+        ylength = abs(ylim[1]) + (abs(ylim[0]) if ylim[0] < 0 else 0)
 
     # # Now calculate stats
     # # # get empty dataframe in case nothing can be calculated
@@ -334,6 +402,9 @@ def swarmboxenplot(
     else:
         pcol = "p-unc"
 
+    if not plot:
+        return stat
+
     # Plot
     # # This ensures there is a point for each `x` class and keep the order correct for below
     mm = data.groupby([x] + ([hue] if hue is not None else []))[y].median()
@@ -403,7 +474,6 @@ def swarmboxenplot(
 def _add_transparency_to_plot(
     ax: Axis, alpha: float = 0.25, kind: str = "boxen"
 ) -> None:
-
     objs = (
         (
             matplotlib.collections.PatchCollection,
